﻿using System;
using System.Web.UI.WebControls;
using DicomObjects;
using DicomObjects.Enums;

namespace DicomWebViewerApplication
{
    public partial class Page2 : System.Web.UI.Page
    {

        private const string NONE = "NONE";
        private const string JPEG = "JPEG";
        private const string PNG = "PNG";

        protected void Page_Load(object sender, EventArgs e)
        {
            if (DicomWebViewer1.Images.Count == 0)
            {
                Reset();
            }
        }
 
        DicomQuery Query()
        {
            DicomQuery query = new DicomQuery
            {
                Node = TextBox2.Text,// "DicomServer.co.uk";
                Port = 104,
                Root = QueryRoot.Patient
            };
            return query;
        }

        private void DoQuery(string keys, TreeNodeCollection collection)
        {
            DicomQuery query = MakeQuery(keys);
            query.Level++;

            if (query.Level == QueryLevel.PATIENT)
                query.Name = TextBox1.Text;

            var results = query.Find();
            foreach (DicomDataSet result in results)
            {
                string name = "!!!";
                if (query.Level == QueryLevel.PATIENT)
                    name = result.Name;
                if (query.Level == QueryLevel.STUDY)
                    name = result.StudyDescription ?? result.StudyUID;
                if (query.Level == QueryLevel.SERIES)
                    name = result.SeriesDescription ?? result.SeriesUID;
                if (query.Level == QueryLevel.IMAGE)
                    name = result.InstanceUID;
                TreeNode TopNode = new TreeNode(name);

                if (query.Level != QueryLevel.IMAGE)
                    TopNode.PopulateOnDemand = true;

                TopNode.Value = $"{query.Level}/{result.PatientID}/{result.StudyUID}/{result.SeriesUID}/{result.InstanceUID}";
                collection.Add(TopNode);
            }
        }

        private DicomQuery MakeQuery(string keys)
        {
            DicomQuery query = Query();
            string[] data = (keys + "/////").Split('/');
            query.Level = (QueryLevel)Enum.Parse(typeof(QueryLevel), data[0]);

            query.PatientID = data[1];
            query.StudyUID = data[2];
            query.SeriesUID = data[3];
            query.InstanceUID = data[4];
            return query;
        }

        protected void TreeView1_SelectedNodeChanged(object sender, EventArgs e)
        {
            DicomQuery query = MakeQuery(TreeView1.SelectedNode.Value);
            DicomDataSetCollection images = query.Get();
            DicomWebViewer1.Images.Clear();

            DicomWebViewer1.MultiColumns = (int)Math.Ceiling(Math.Sqrt(images.Count));
            DicomWebViewer1.MultiRows = (int)Math.Ceiling((double)images.Count/DicomWebViewer1.MultiColumns);
            DicomWebViewer1.Images.Add(images);
        }

        protected void TreeView1_TreeNodePopulate(object sender, TreeNodeEventArgs e)
        {
            TreeNodeCollection collection = e.Node.ChildNodes;
            DoQuery(e.Node.Value, collection);
        }

        protected void Button1_Click(object sender, EventArgs e)
        {
            TreeView1.Nodes.Clear();
            DoQuery(NONE, TreeView1.Nodes);
        }

        protected void Button2_Click(object sender, EventArgs e)
        {
            Reset();
        }

        private void Reset()
        {
            DicomWebViewer1.MultiColumns = 2;
            DicomWebViewer1.MultiRows = 2;
            DicomWebViewer1.Images.Clear();

            DicomWebViewer.DicomWebViewer.HtmlDicomImage mammo1 = new DicomWebViewer.DicomWebViewer.HtmlDicomImage(Resource1.mammo)
            {
                Format = JPEG
            };

            DicomWebViewer1.Images.Add(mammo1);

            DicomWebViewer.DicomWebViewer.HtmlDicomImage mammo2 = new DicomWebViewer.DicomWebViewer.HtmlDicomImage(Resource1.mammo)
            {
                Format = PNG
            };

            DicomWebViewer1.Images.Add(mammo2);

            DicomWebViewer1.Images.Read(Resource1.CT1);
            DicomWebViewer1.Images.Read(Resource1.im1);
            DicomWebViewer1.CurrentIndex = 0;            
        }
    }
}