﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Routing;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web.Script.Serialization;
using System.Drawing.Drawing2D;
using System.Web.SessionState;
using DicomObjects.Enums;
using DicomObjects;
using System.Globalization;

namespace DicomWebViewer
{
    [DefaultProperty("Text")]
    [ToolboxData("<{0}:DicomWebViewer class=\"DicomWebViewer\" runat=server></{0}:DicomWebViewer>")]
    public class DicomWebViewer : WebControl, IRouteHandler, IHttpHandler, IReadOnlySessionState
    {

        private const string ImagesText = "Images";
        private const string GuidText = "Guid";
        private const string Jquery = "jquery";
        private const string DicomWebViewer_JqueryMin = "DicomWebViewer.jquery.min.js";
        private const string JsDicomViewer = "jsDicomViewer";
        private const string DicomWebViewer_JsDicomViewer = "DicomWebViewer.jsDicomViewer.js";
        private const string MultiRowsText = "MultiRows";
        private const string MultiColumnsText = "MultiColumns";
        private const string CurrentIndexText = "CurrentIndex";
        private const string JPEG = "JPEG";
        private const string JPG = "JPG";
        private const string PNG = "PNG";
        private const string BMP = "BMP";
        private const string TIFF = "TIFF";
        private const string GIF = "GIF";
        private const string ContentType = "application/json; charset=utf-8";
        private const string OperationText = "Operation";
        private const string Viewer = "VIEWER";
        private const string Header_CacheControl_Text = "Cache-Control";
        private const string Header_CacheControl_Value = "no-cache";
        private const string RequestParams_ImageID = "ImageID";
        private const string RequestParams_CellWidth = "CellWidth";
        private const string RequestParams_CellHeight = "CellHeight";
        private const string RequestParams_Width = "Width";
        private const string RequestParams_Level = "Level";
        private const string RequestParams_ScrollX = "ScrollX";
        private const string RequestParams_ScrollY = "ScrollY";
        private const string RequestParams_Zoom = "Zoom";
        private const string RequestParams_Text = "Text";
        private const string RequestParams_Downsample = "Downsample";
        private const string Data = "DATA";

        private DicomImageCollection m_Images;
        public DicomImageCollection Images
        {
            get
            {
                m_Images = GetData(ImagesText) as DicomImageCollection;
                if (m_Images == null)
                {
                    m_Images = new DicomImageCollection();
                    SaveData(ImagesText, m_Images);
                }
                return m_Images;
            }
        }

        private Guid m_Guid;
        public Guid Guid
        {
            get
            {
                if (GetData(GuidText) == null)
                {
                    m_Guid = Guid.NewGuid();

                    SaveData(GuidText, m_Guid);
                    SetRoute();
                }
                else
                {
                    m_Guid = (Guid)GetData(GuidText);
                }
                return m_Guid;
            }
        }
        
        private void SetRoute()
        {
            Route route = new Route(Guid.ToString() + "/{Operation}", this);
            RouteTable.Routes.Add(route);
        }

        protected override void OnPreRender(EventArgs e)
        {
            string name = ClientID;
            ClientScriptManager cs = Page.ClientScript;

            // hosted version
            //cs.RegisterClientScriptInclude("JQUERYFULL", "http://ajax.googleapis.com/ajax/libs/jquery/1.8.1/jquery.js");
            
            // embedded
            if (!Page.ClientScript.IsClientScriptIncludeRegistered(Jquery))
                Page.ClientScript.RegisterClientScriptInclude(Jquery, Page.ClientScript.GetWebResourceUrl(GetType(), DicomWebViewer_JqueryMin));
            
            if (!Page.ClientScript.IsClientScriptIncludeRegistered(JsDicomViewer))
                Page.ClientScript.RegisterClientScriptInclude(JsDicomViewer, Page.ClientScript.GetWebResourceUrl(GetType(), DicomWebViewer_JsDicomViewer));
            
            string Loader = "$.getJSON('" + Guid.ToString() + "/VIEWER" + "?unique=" + DateTime.Now.Ticks.ToString() +
                "', function(data){  })" +
                ".success(function(result) {GotData_" + name + "(result);});" + "\r\n";

            string code = //"alert('XYZ1'); " + "\r\n" +
            "$(document).ready(function () {" + "\r\n" +
                 Loader +
            "});" + "\r\n" +
            "function GotData_" + name + "(result) {" + "\r\n" +
                "DicomImageViewer.SetupRender(\"" + name + "\", result); " + "\r\n" +
            //"alert('ABC1'); " + "\r\n" +
            "}" + "\r\n"; 

            cs.RegisterClientScriptBlock(GetType(), name, code, true);
            base.OnPreRender(e);
        }

        void SaveData(string name, object data)
        {
            HttpContext.Current.Session[this.ID + "/" + name] = data;
        }

        object GetData(string name)
        {
            return HttpContext.Current.Session[this.ID + "/" + name];
        }

        private int m_MultiRows = 1;
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue(1)]
        [Localizable(true)]
        public int MultiRows
        {
            get
            {
                if (GetData(MultiRowsText) == null)
                {
                    m_MultiRows = 1;
                    SaveData(MultiRowsText, m_MultiRows);
                }
                else
                {
                    m_MultiRows = (int)GetData(MultiRowsText);
                }
                return m_MultiRows;
            }
            set
            {
                m_MultiRows = value;
                SaveData(MultiRowsText, m_MultiRows);
            }
        }
        private int m_MultiColumns = 1;

        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue(1)]
        [Localizable(true)]
        public int MultiColumns
        {
            get
            {
                if (GetData(MultiColumnsText) == null)
                {
                    m_MultiColumns = 1;
                    SaveData(MultiColumnsText, m_MultiColumns);
                }
                else
                {
                    m_MultiColumns = (int)GetData(MultiColumnsText);
                }
                return m_MultiColumns;
            }
            set
            {
                m_MultiColumns = value;
                SaveData(MultiColumnsText, m_MultiColumns);
            }
        }

        public class HtmlDicomImage : DicomImage
        {
            public HtmlDicomImage(string filename) : base(filename) { }
            public HtmlDicomImage(Stream filename) : base(filename) { }
            public HtmlDicomImage(byte[] filename) : base(filename) { }
            public string Format { get; set; }

        }

        class JsDicomWebViewerData
        {
            public List<JsDicomImage> images = new List<JsDicomImage>();
            public int multiRows, multiColumns;
            public int currentIndex;            
            public Guid guid;            
            public int totalImages;            

            public JsDicomWebViewerData(DicomWebViewer viewer)
            {
                guid = viewer.Guid;
                multiRows = viewer.MultiRows;
                multiColumns = viewer.MultiColumns;
                currentIndex = viewer.CurrentIndex;                
                int n = 0;
                foreach (var image in viewer.Images)
                {
                    JsDicomImage jsImage = new JsDicomImage(image);
                    jsImage.n = n++;
                    images.Add(jsImage);
                }
                totalImages = viewer.Images.Count;
            }

            internal string JsonData()
            {
                var j = new JavaScriptSerializer();
                return j.Serialize(this);
            }
        }

        private int m_CurrentIndex = 0;

        public int CurrentIndex
        {
            get
            {
                if (GetData(CurrentIndexText) == null)
                {
                    m_CurrentIndex = 0;
                    SaveData(CurrentIndexText, m_CurrentIndex);
                }
                else
                {
                    m_CurrentIndex = (int)GetData(CurrentIndexText);
                }
                return m_CurrentIndex;
            }
            set
            {
                m_CurrentIndex = value;
                SaveData(CurrentIndexText, m_CurrentIndex);
            }
        }
        
        public IHttpHandler GetHttpHandler(RequestContext requestContext) { return this; }

        public bool IsReusable
        {
            get { return true; }
        }

        class Pixels
        {
            public int h;
            public int w;
            public Array p;
            public float width, level;
            public int downsample;
            public string id;

            private readonly DicomImage Image;

            public Pixels(DicomImage Image, int downsample)
            {
                // TODO: Complete member initialization
                this.Image = Image;
                this.downsample = downsample;
                w = Image.Size.Width / downsample;
                h = Image.Size.Height / downsample;
                width = Image.Width;
                level = Image.Level;
                id = Image.InstanceUID;
            }
        }

        public DicomImage GetImage(string id)
        {
            return Images[id];
        }

        private ImageFormat ImageFormatFromString(string format)
        {
            switch (format.ToUpper())
            {
                case JPEG:
                case JPG:
                    return ImageFormat.Jpeg;
                case PNG:
                    return ImageFormat.Png;
                case BMP:
                    return ImageFormat.Bmp;
                case TIFF:
                    return ImageFormat.Tiff;
                case GIF:
                    return ImageFormat.Gif;
                default:
                    return null;
            }
        }

        public void ProcessRequest(HttpContext context)
        {
            string Operation = context.Request.RequestContext.RouteData.Values[OperationText] as string;
            if (Operation == Viewer)
            {
                var cont = new JsDicomWebViewerData(this);
                context.Response.AddHeader(Header_CacheControl_Text, Header_CacheControl_Value);
                context.Response.ContentType = ContentType;
                context.Response.Output.Write(cont.JsonData());
                return;
            }

            string id = context.Request.Params[RequestParams_ImageID];
            int CellWidth = (int)float.Parse(context.Request.Params[RequestParams_CellWidth], CultureInfo.InvariantCulture.NumberFormat);
            int CellHeight = (int)float.Parse(context.Request.Params[RequestParams_CellHeight], CultureInfo.InvariantCulture.NumberFormat);

            DicomImage Image = GetImage(id);
            ImageFormat Format = ImageFormatFromString(Operation);

            if (Format != null)
            {
                if (Image != null) //ImageNumber < Images.Count)
                {
                    Image.Width = float.Parse(context.Request.Params[RequestParams_Width], CultureInfo.InvariantCulture.NumberFormat);
                    Image.Level = float.Parse(context.Request.Params[RequestParams_Level], CultureInfo.InvariantCulture.NumberFormat);
                    Image.Scroll = Point.Round(new PointF(float.Parse(context.Request.Params[RequestParams_ScrollX], CultureInfo.InvariantCulture.NumberFormat), 
                    float.Parse(context.Request.Params[RequestParams_ScrollY], CultureInfo.InvariantCulture.NumberFormat)));
                    Image.Zoom = float.Parse(context.Request.Params[RequestParams_Zoom].Substring(0, Math.Min(8, context.Request.Params[RequestParams_Zoom].Length)), CultureInfo.InvariantCulture.NumberFormat);
                    Image.StretchMode = StretchModes.NoStretch;

                    context.Response.ContentType = "image/" + Format.ToString().ToLower();
                    Size sz = new Size(CellWidth, CellHeight);
                    Matrix matrix = Image.Matrix(sz);

                    var text = context.Request.Params[RequestParams_Text];
                    if (text != null)
                    {
                        DicomLabel label = new DicomLabel()
                        {
                            Text = text,
                            TextBrush = Brushes.Red,
                            Area = new RectangleF(0.2F, 0.2F, 0.6F, 0.6F),
                            LabelType = LabelType.Text,
                            ScaleMode = ScaleMode.Cell,
                            Font = new Font(FontFamily.GenericSansSerif, 50)
                        };
                        Image.Labels.Add(label);
                    }

                    Bitmap bitmap = Image.Bitmap(sz, PixelFormat.Format24bppRgb, matrix);

                    MemoryStream memoryStream = new MemoryStream();

                    bitmap.Save(memoryStream, Format);
                    memoryStream.Position = 0;
                    memoryStream.CopyTo(context.Response.OutputStream);

                    if (text != null)
                    {
                        Image.Labels.RemoveAt(Image.Labels.Count - 1);
                    }

                }
                return;
            }
            switch (Operation.ToUpper())
            {
                case Data:
                    {
                        context.Response.ContentType = ContentType;

                        int Downsample = int.Parse(context.Request.Params[RequestParams_Downsample], CultureInfo.InvariantCulture.NumberFormat);

                        Pixels p = new Pixels(Image, Downsample);
                        var j = new JavaScriptSerializer();


                        int[] r = new int[p.w * p.h];
                        int k = 0;
                        if ((ushort)(Image.DataSet[Keyword.BitsAllocated].Value) <= 8)
                        {
                            byte[, ,] data = Image.Pixels as byte[, ,];
                            for (int y = 0; y < p.h; y++)
                            {
                                for (int x = 0; x < p.w; x++)
                                {
                                    r[k++] = data[x * Downsample, y * Downsample, 0];
                                }
                            }
                        }
                        else if ((ushort)(Image.DataSet[Keyword.PixelRepresentation].Value) > 0)  // signed
                        {
                            short[, ,] data = Image.Pixels as short[, ,];


                            for (int y = 0; y < p.h; y++)
                            {
                                for (int x = 0; x < p.w; x++)
                                {
                                    r[k++] = data[x * Downsample, y * Downsample, 0];
                                }
                            }
                        }
                        else if ((ushort)(Image.DataSet[Keyword.PixelRepresentation].Value) == 0)  // unsigned
                        {
                            ushort[, ,] data = Image.Pixels as ushort[, ,];

                            for (int y = 0; y < p.h; y++)
                            {
                                for (int x = 0; x < p.w; x++)
                                {
                                    r[k++] = data[x * Downsample, y * Downsample, 0];
                                }
                            }
                        }
                        p.p = r;

                        j.MaxJsonLength = int.MaxValue;

                        string s = j.Serialize(p);

                        context.Response.Output.Write(s);
                    }
                    break;
            }

        }
    }
}
